xquery version "3.1";

import module namespace via="http://dev.met.de/via/main" at "xmldb:exist:///db/apps/via/modules/via.xqm";
import module namespace trivia="http://dev.met.de/via/trivia" at "xmldb:exist:///db/apps/via/modules/trivia.xqm";
import module namespace config="http://dev.met.de/via/config" at "xmldb:exist:///db/apps/via/modules/config.xqm";
import module namespace console="http://exist-db.org/xquery/console";
import module namespace tgclient="https://sade.textgrid.de/ns/tgclient" at "xmldb:exist:///db/apps/via/modules/tgclient.xqm";
declare default element namespace "http://dev.met.de/via/";
declare namespace vias="http://dev.met.de/via/schema";
declare namespace svg="http://www.w3.org/2000/svg";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xlink="http://www.w3.org/1999/xlink";
declare namespace tg="http://textgrid.info/namespaces/metadata/core/2010";


declare namespace output = "http://www.w3.org/2010/xslt-xquery-serialization";
declare option exist:serialize "method=json media-type=application/json";


declare function local:analyse-trivia-data
    ( $trivia-json-object as xs:string ) as item()*{
    
    let $trivia-map := parse-json($trivia-json-object)
    for $trivia-surface in $trivia-map?*[1]
    let $surface-metadata := trivia:get-trivia-surface-metadata( $trivia-surface )
    return
        if ( $surface-metadata?file_attributes?imageRef and $surface-metadata?file_attributes?textRef ) then (
            (:trivia:create-trivia-xml-container-objects( $trivia-surface ):)
            (:local:create-trivia-xml-resource($trivia-surface):)
            (:trivia:create-tei-xml-resource( $trivia-surface ):)
            (:local:update-tei-xml-resource( local:create-tei-xml-resource( $trivia-surface ) , $trivia-surface):)

        ) else if ( $surface-metadata?file_attributes?imageRef and $surface-metadata?file_attributes?textRef = '' ) then (
            "create tei zone in TextGrid"
        ) else (
            'no referenzes set'
        )
};


declare function local:process-trivia-data
    ( $trivia-json-object as map(*) ) as item()*{
    let $array := array{
        for $trivia-surface-id in map:keys( $trivia-json-object )    
        
        let $trivia-surface := local:process-trivia-surface( $trivia-json-object( $trivia-surface-id) )
        return (
            map{ "uri" : $trivia-surface-id, "data" : $trivia-surface}
        )
    }
    return $array
};

declare function local:process-trivia-surface
    ( $trivia-surface as map(*) ) as item()*{
        let $surface-metadata := trivia:get-trivia-surface-metadata( $trivia-surface )
        let $trivia-xml := trivia:create-trivia-xml-resource-from-trivia-surface( $trivia-surface ) 
        let $tei-xml := trivia:create-tei-xml-resource-from-trivia-surface( $trivia-surface )
        
        let $result := map{ 'trivia' : $trivia-xml, 'tei' : $tei-xml}
        return $result
    };


declare function local:send-trivia-data-to-textgrid
    ( $trivia-json-object as map(*), $mode as xs:string ) as item()*{
        let $resources := local:process-trivia-data( $trivia-json-object )
        for $resource in $resources?*
        return 
            trivia:update-trivia-xml-within-textgrid( $via:tg-config, $resource?uri, $resource?data?trivia )
            
    };
    
    
declare function local:update-resource-or-create-if-not-existing
    ( $config as map(*), $uri as xs:string, $data as node(), $metadata as map(*)? ) as item()* {
    
    let $title := $metadata?title
    let $format := $metadata?format
    let $parsed-uri := tgclient:parse-uri( $uri )
    let $requested-resource := tgclient:get-resource($config, $uri)
    return 
        if ( $requested-resource?data ) then (
            (: update resource :)
            let $create-revision := tgclient:revision( $config?sid, $config?tgcrudURL, $data, $requested-resource?metadata )
            
            (: return resource-uri :)
            return $create-revision//*:generic//*:textgridUri/string()
        ) else (
            (: create resource :)
            (:let $create-resource :=  tgclient:create-resource($config, $title, $format, $resource):)
            
            (: identify resource-uri :)
            (:let $new-objects-uri := tgclient:parse-uri( $create-resource//*:generic//*:textgridUri/string() )("prefixed-base"):)
            
            (: return object-uri :)
            (:return $new-objects-uri:)
        )
    (:
    let $create-resource :=  tgclient:create-resource($config, $title, $format, $resource)
    let $new-objects-uri := tgclient:parse-uri( $create-resource//*:generic//*:textgridUri/string() )("prefixed-base")
    let $push := tgclient:push-to-aggregation($config, $new-objects-uri, $aggregation-uri)
    let $resource := tgclient:get-resource($config, $new-objects-uri)
    return $resource
    :)
};

declare function local:create-tei-resource
    ( $trivia-json-object as xs:string ) as item()*{
    
    let $trivia-map := parse-json($trivia-json-object)
    for $trivia-surface in $trivia-map?*[1]
    let $surface-metadata := trivia:get-trivia-surface-metadata( $trivia-surface )
    return
        if ( $surface-metadata?file_attributes?imageRef and $surface-metadata?file_attributes?textRef ) then (
            (:trivia:create-trivia-xml-container-objects( $trivia-surface ):)
            (:local:create-trivia-xml-resource($trivia-surface):)
            (:trivia:create-tei-xml-resource( $trivia-surface ):)
            (:local:update-tei-xml-resource( local:create-tei-xml-resource( $trivia-surface ) , $trivia-surface):)

        ) else if ( $surface-metadata?file_attributes?imageRef and $surface-metadata?file_attributes?textRef = '' ) then (
            "create tei zone in TextGrid"
        ) else (
            'no referenzes set'
        )
};

(: ################################################################################################################# :)
(:                                          Testcenter   TEI > Trivia                                                :) 
(: ################################################################################################################# :)

declare function local:convert-xml-resources-to-trivia-surface
    ( $tei-xml as node(), $trivia-xml as node() ) as item()*{
    
    let $svg-shapes := $trivia-xml//svg:svg/svg:g/node()[ not(self::svg:image) ]
    let $zone-structure := $tei-xml//tei:sourceDoc/tei:surface
    let $trivia-shapes := local:convert-tei-zones-to-trivia-shapes( $zone-structure/tei:zone, $svg-shapes )
    
    
    let $filename := $trivia-xml//metadata/title
    let $size := $trivia-xml//metadata/size
    let $attributes := $trivia-xml//metadata/attributes/node()
    let $file-attributes := via:xml-to-map( $attributes )
    let $file-attributes := if ( $trivia-xml//svg:image/@height ) then ( 
            via:map-put( $file-attributes, "naturalHeight", number( data($trivia-xml//svg:image/@height) ) ) 
        ) else ( $file-attributes )
    let $file-attributes := if ( $trivia-xml//svg:image/@width ) then (
            via:map-put( $file-attributes, "naturalWidth", number( data($trivia-xml//svg:image/@width) ) )
        ) else ( $file-attributes )
        
    return 
        map{
            "filename" : "",
            "size" : number( data($size) ),
            "file_attributes" : $file-attributes,
            "regions" : array{ $trivia-shapes }
        }
        
};

declare function local:convert-tei-zones-to-trivia-shapes
    ( $tei-xml-zones as node()*, $svg-xml-shapes as node()* ) as item()*{
    
    for $zone in $tei-xml-zones
    let $id := data($zone/@xml:id)
    let $name := data($zone/@n)
    let $mapItem := data($zone/@type)
    let $shape := $svg-xml-shapes[@id = $id]
    let $region-attributes := $zone/tei:note/tei:fs/tei:f
    let $child-zones := $zone/tei:zone
    return(
        map{
            "region_attributes" : map:new ( 
                let $map := map{
                        "zoneId": $id ,
                        "name": if ($name) then ($name) else ("")
                    }
                let $map := map:new ((
                        $map, 
                        for $att in $region-attributes
                        return map:entry( data($att/@name), data( $att/node() ) )
                    ))
                return $map
                ),
            "shape_attributes" : map:new (
                        
                let $map := map{"name" : substring-after($shape/name(), ":")} 
                let $map := map:new ( 
                    (
                        $map, 
                        for $att in $shape/@*
                        where $att/name() != "id" and $att/name() != ''
                        return map:entry( $att/name(), data($att) )
                    ) 
                )
                return trivia:convert-shape-to-via-shape( $map )
            )
        },
        if ( $child-zones ) then (
            local:convert-tei-zones-to-trivia-shapes($child-zones, $svg-xml-shapes)
        ) else ()
    )
};


declare function local:build-trivia-object-from-textgrid-resources
    ( $trivia-resource-uri as xs:string, $sid as xs:string ) as item()* {
    
    let $config := map{ "sid" : $sid, "tgcrudURL" : $via:tg-config?tgcrudURL }
    let $trivia-map := tgclient:get-resource($config, $trivia-resource-uri)
    let $metadata := $trivia-map?metadata
    let $uri := $trivia-map?uri
    let $trivia-xml := $trivia-map?data
    
    let $linked-tei-uri := data( $trivia-map?data//textRef )
    let $tei-xml := tgclient:get-resource($config, $linked-tei-uri)?data

    return 
        if ( $trivia-xml/trivia and $tei-xml ) then (
            map{
                "uri"   : $uri,
                "title" : data( $metadata//tg:generic/tg:provided/tg:title ),
                "object": local:convert-xml-resources-to-trivia-surface( $tei-xml, $trivia-xml )
            }
        ) else (
            element {"error"}{"resource is not a trivia resource!"}
        )
    
}; 


(: First Try :)
(:
let $parsed := parse-json( $via:json-sample ) 
let $image := via:get-via-surfaces($parsed)[2]
let $parsed-zones := zones:parse-json( $zones:sample )
let $shape-object := via:convert-via-surface-to-shapes-surface($image)
let $zones := zones:correlate-zones-with-shapes-object($shape-object, $parsed-zones, "textgrid:triv123")
let $tei-sourcedoc := zones:convert-to-tei-sourceDoc($zones)
let $tei-conv := zones:convert-to-tei-resource($tei-sourcedoc, ())
:)

(: Second Try on real trivia data :)

(: let $fake := "textgrid:triv123"
let $no-fake := "textgrid:3r346":)

(: let $analysed-trivia-objects := local:analyse-trivia-data($trivia:json2)
let $tg-resource := tgclient:get-data("textgrid:triv123", $via:tg-config("tgcrudURL"), $via:tg-config("sid") )
:)
(:  :let $res := trivia:get-trivia-object-from-textgrid( "textgrid:3r3vv", $via:tg-config?sid ):)
(:  :let $update := local:update-resource-or-create-if-not-existing($via:tg-config, $no-fake, $res?data, map{'title' : 'my new test', 'format' : 'text/xml'}):)
(:  :return $res:)


(: from trivia to textgrid :)
(:
let $json := parse-json( $trivia:real-json2 )
let $processed := local:send-trivia-data-to-textgrid($json, "update")
return $processed
:)

let $surface := parse-json($trivia:export-sample)
let $trivia-zones := $surface?zones
let $tei := trivia:get-referenced-text-resource-from-trivia-surface-via-textgrid( $surface, $via:tg-config("sid") )
let $updated-tei := trivia:update-tei-xml-resource-from-trivia-surface($tei, $surface)
(:  :let $get :=trivia:build-trivia-object-from-textgrid-resources( "textgrid:3r3p5", $via:tg-config?sid):)
(:  :let $zones := trivia:create-zones-xml-from-trivia-zones( $trivia-zones )
let $source-doc := trivia:create-tei-sourceDoc-node-from-trivia-surface( $surface, $zones )
let $update := trivia:replace-sourcedoc( $tei, <b/> )
:)
(:  :let $trivia := trivia:create-trivia-xml-resource-from-trivia-surface( $surface ) :)
(:  :let $convert-tei := local:trivia-data-object-from-trivia-and-tei-xml( $trivia, $tei, "uri", <title/> ):)
(:  :return ($convert-tei, $tei, $trivia):)

(:  :let $get :=trivia:build-trivia-object-from-textgrid-resources( "textgrid:3r3p5", $via:tg-config?sid):)
return $updated-tei
